@@ -29,6 +29,10 @@ def is_bigquery(engine: sa.engine.Engine) -> bool:
     return engine.name == "bigquery"
 
 
+def is_impala(engine: sa.engine.Engine) -> bool:
+    return engine.name == "impala"
+
+
 def get_table_columns(table, column_names):
     return [table.c[column_name] for column_name in column_names]
 
@@ -408,6 +412,15 @@ def get_date_span(engine, ref, date_column_name):
                 )
             ]
         )
+    elif is_impala(engine):
+        selection = sa.select(
+            [
+                sa.func.datediff(
+                    sa.func.to_date(sa.func.max(column)),
+                    sa.func.to_date(sa.func.min(column)),
+                )
+            ]
+        )
     else:
         raise NotImplementedError(
             "Date spans not yet implemented for this sql dialect."
@@ -620,6 +633,14 @@ def get_date_gaps(
             )
             > legitimate_gap_size
         )
+    elif is_impala(engine):
+        gap_condition = (
+            sa.func.datediff(
+                sa.func.to_date(start_table.c[start_column]),
+                sa.func.to_date(end_table.c[end_column]),
+            )
+            > legitimate_gap_size
+        )
     elif is_bigquery(engine):
         # see https://cloud.google.com/bigquery/docs/reference/standard-sql/date_functions#date_diff
         # Note that to have a gap (positive date_diff), the first date (start table)
@@ -728,6 +749,8 @@ def get_max(engine, ref):
 
 def get_mean(engine, ref):
     def column_operator(column):
+        if is_impala(engine):
+            return sa.func.avg(column)
         return sa.func.avg(sa.cast(column, sa.DECIMAL))
 
     return get_column(engine, ref, aggregate_operator=column_operator)
@@ -901,11 +924,15 @@ def get_row_mismatch(engine, ref, ref2, match_and_compare):
 
     avg_match_column = sa.func.avg(sa.case([(compare, 0.0)], else_=1.0))
 
-    selection = sa.select([avg_match_column]).select_from(
+    selection_difference = sa.select([avg_match_column]).select_from(
         subselection1.join(subselection2, match)
     )
-    result = engine.connect().execute(selection).scalar()
-    return result, [selection]
+    selection_n_rows = sa.select(sa.func.count()).select_from(
+        subselection1.join(subselection2, match)
+    )
+    result_mismatch = engine.connect().execute(selection_difference).scalar()
+    result_n_rows = engine.connect().execute(selection_n_rows).scalar()
+    return result_mismatch, result_n_rows, [selection_difference, selection_n_rows]
 
 
 def get_duplicate_sample(engine, ref):
@@ -1124,9 +1151,14 @@ def get_regex_violations(engine, ref, aggregated, regex, n_counterexamples):
     if aggregated:
         subquery = subquery.distinct()
     subquery = subquery.subquery()
-    violation_selection = sa.select(subquery.c[column]).where(
-        sa.not_(subquery.c[column].regexp_match(regex))
-    )
+    if is_impala(engine):
+        violation_selection = sa.select(subquery.c[column]).where(
+            sa.not_(sa.func.regexp_like(subquery.c[column], regex))
+        )
+    else:
+        violation_selection = sa.select(subquery.c[column]).where(
+            sa.not_(subquery.c[column].regexp_match(regex))
+        )
     n_violations_selection = sa.select([sa.func.count()]).select_from(
         violation_selection.subquery()
     )
