@@ -6,8 +6,9 @@ import urllib.parse
 
 import pytest
 import sqlalchemy as sa
+from impala.dbapi import connect
 
-from datajudge.db_access import apply_patches, is_bigquery, is_mssql
+from datajudge.db_access import apply_patches, is_bigquery, is_impala, is_mssql
 
 TEST_DB_NAME = "tempdb"
 SCHEMA = "dbo"  # 'dbo' is the standard schema in mssql
@@ -15,6 +16,18 @@ SCHEMA = "dbo"  # 'dbo' is the standard schema in mssql
 
 def get_engine(backend) -> sa.engine.Engine:
     address = os.environ.get("DB_ADDR", "localhost")
+
+    if backend == "impala":
+
+        def conn_creator():
+            return connect(
+                host=address,
+                port=21050,
+                database="default",
+            )
+
+        return sa.create_engine("impala://", creator=conn_creator)
+
     if backend == "postgres":
         connection_string = f"postgresql://datajudge:datajudge@{address}:5432/datajudge"
     elif "mssql" in backend:
@@ -34,8 +47,8 @@ def get_engine(backend) -> sa.engine.Engine:
         account = os.environ.get("SNOWFLAKE_ACCOUNT", "")
         connection_string = f"snowflake://{user}:{password}@{account}/datajudge/DBO?warehouse=datajudge&role=accountadmin"
     elif "bigquery" in backend:
-        gcp_project = os.environ.get("GOOGLE_CLOUD_PROJECT", "scratch-361908")
-        connection_string = f"bigquery://{gcp_project}"
+        # gcp_project = os.environ.get("GOOGLE_CLOUD_PROJECT", "scratch-361908")
+        connection_string = "bigquery://"
 
     engine = sa.create_engine(connection_string, echo=True)
     apply_patches(engine)
@@ -47,7 +60,7 @@ def get_engine(backend) -> sa.engine.Engine:
 def engine(backend):
     engine = get_engine(backend)
     with engine.connect() as conn:
-        if engine.name in ("postgresql", "bigquery"):
+        if engine.name in ("postgresql", "bigquery", "impala"):
             conn.execute(f"CREATE SCHEMA IF NOT EXISTS {SCHEMA}")
     return engine
 
@@ -78,7 +91,6 @@ def int_table1(engine, metadata):
 @pytest.fixture(scope="module")
 def int_table2(engine, metadata):
     table_name = "int_table2"
-    engine.execute(f"DROP TABLE IF EXISTS {SCHEMA}.{table_name}")
     columns = [sa.Column("col_int", sa.Integer())]
     data = [{"col_int": i} for i in range(2, 20)]
     _handle_table(engine, metadata, table_name, columns, data)
@@ -88,7 +100,6 @@ def int_table2(engine, metadata):
 @pytest.fixture(scope="module")
 def int_square_table(engine, metadata):
     table_name = "int_square_table"
-    engine.execute(f"DROP TABLE IF EXISTS {SCHEMA}.{table_name}")
     columns = [sa.Column("col_int", sa.Integer())]
     data = [{"col_int": i**2} for i in range(1, 20)]
     _handle_table(engine, metadata, table_name, columns, data)
@@ -136,6 +147,26 @@ def mix_table2(engine, metadata):
     return TEST_DB_NAME, SCHEMA, table_name
 
 
+@pytest.fixture(scope="module")
+def mix_table2_pk(engine, metadata):
+    table_name = "mix_table2_pk"
+    columns = [
+        sa.Column("col_int", sa.Integer(), primary_key=True),
+        sa.Column("col_varchar", sa.String()),
+        sa.Column("col_date", sa.DateTime()),
+    ]
+    data = [
+        {
+            "col_int": i,
+            "col_varchar": f"hi{i}",
+            "col_date": datetime.datetime(2016, 1, i // 2),
+        }
+        for i in range(2, 20)
+    ]
+    _handle_table(engine, metadata, table_name, columns, data)
+    return TEST_DB_NAME, SCHEMA, table_name
+
+
 @pytest.fixture(scope="module")
 def date_table1(engine, metadata):
     table_name = "date_table1"
@@ -675,7 +706,7 @@ def random_normal_table(engine, metadata):
     Table with normally distributed values of varying means and sd 1.
     """
 
-    if is_bigquery(engine):
+    if is_bigquery(engine) or is_impala(engine):
         # It takes too long to insert the table into BigQuery,
         # test using this fixture must be disabled for BigQuery
         return None, None, None
@@ -719,6 +750,10 @@ def capitalization_table(engine, metadata):
     elif is_bigquery(engine):
         str_datatype = "STRING"
         primary_key = ""  # there is no primary key in BigQuery
+    elif is_impala(engine):
+        str_datatype = "STRING"
+        # Impala supports primary keys but uses a different grammar.
+        primary_key = ""
     else:
         str_datatype = "TEXT"
     with engine.connect() as connection:
@@ -760,7 +795,9 @@ def cross_cdf_table2(engine, metadata):
 def pytest_addoption(parser):
     parser.addoption(
         "--backend",
-        choices=(("mssql", "mssql-freetds", "postgres", "snowflake", "bigquery")),
+        choices=(
+            ("mssql", "mssql-freetds", "postgres", "snowflake", "bigquery", "impala")
+        ),
         help="which database backend to use to run the integration tests",
     )
 
